@@ -1,7 +1,7 @@
 ;;; gnus-agent.el --- unplugged support for Gnus
 ;; Copyright (C) 1997,98 Free Software Foundation, Inc.
 
-;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
+;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
@@ -54,8 +54,31 @@
   :group 'gnus-agent
   :type 'integer)
 
+(defcustom gnus-agent-expire-all nil
+  "If non-nil, also expire unread, ticked and dormant articles.
+If nil, only read articles will be expired."
+  :group 'gnus-agent
+  :type 'boolean)
+
+(defcustom gnus-agent-group-mode-hook nil
+  "Hook run in Agent group minor modes."
+  :group 'gnus-agent
+  :type 'hook)
+
+(defcustom gnus-agent-summary-mode-hook nil
+  "Hook run in Agent summary minor modes."
+  :group 'gnus-agent
+  :type 'hook)
+
+(defcustom gnus-agent-server-mode-hook nil
+  "Hook run in Agent summary minor modes."
+  :group 'gnus-agent
+  :type 'hook)
+
 ;;; Internal variables
 
+(defvar gnus-agent-meta-information-header "X-Gnus-Agent-Meta-Information")
+
 (defvar gnus-agent-history-buffers nil)
 (defvar gnus-agent-buffer-alist nil)
 (defvar gnus-agent-article-alist nil)
@@ -181,7 +204,8 @@
 						     buffer))))
 	    minor-mode-map-alist))
     (gnus-agent-toggle-plugged gnus-plugged)
-    (gnus-run-hooks 'gnus-agent-mode-hook)))
+    (gnus-run-hooks 'gnus-agent-mode-hook
+		    (intern (format "gnus-agent-%s-mode-hook" buffer)))))
 
 (defvar gnus-agent-group-mode-map (make-sparse-keymap))
 (gnus-define-keys gnus-agent-group-mode-map
@@ -308,19 +332,34 @@ agent minor mode in all Gnus buffers."
     (re-search-forward
      (concat "^" (regexp-quote mail-header-separator) "\n"))
     (replace-match "\n")
+    (gnus-agent-insert-meta-information 'mail)
     (gnus-request-accept-article "nndraft:queue")))
 
+(defun gnus-agent-insert-meta-information (type &optional method)
+  "Insert meta-information into the message that says how it's to be posted.
+TYPE can be either `mail' or `news'.  If the latter METHOD can
+be a select method."
+  (save-excursion
+    (message-remove-header gnus-agent-meta-information-header)
+    (goto-char (point-min))
+    (insert gnus-agent-meta-information-header ": "
+	    (symbol-name type) " " (format "%S" method)
+	    "\n")
+    (forward-char -1)
+    (while (search-backward "\n" nil t)
+      (replace-match "\\n" t t))))
+
 ;;;
 ;;; Group mode commands
 ;;;
 
 (defun gnus-agent-fetch-groups (n)
-  "Put all new articles in the current groups into the agent."
+  "Put all new articles in the current groups into the Agent."
   (interactive "P")
   (gnus-group-iterate n 'gnus-agent-fetch-group))
 
 (defun gnus-agent-fetch-group (group)
-  "Put all new articles in GROUP into the agent."
+  "Put all new articles in GROUP into the Agent."
   (interactive (list (gnus-group-group-name)))
   (unless group
     (error "No group on the current line"))
@@ -364,7 +403,7 @@ agent minor mode in all Gnus buffers."
       (error "Server already in the agent program"))
     (push method gnus-agent-covered-methods)
     (gnus-agent-write-servers)
-    (message "Entered %s into the agent" server)))
+    (message "Entered %s into the Agent" server)))
 
 (defun gnus-agent-remove-server (server)
   "Remove SERVER from the agent program."
@@ -490,6 +529,19 @@ the actual number of articles toggled is returned."
     (when (file-exists-p (gnus-agent-lib-file "active"))
       (delete-file (gnus-agent-lib-file "active"))))
 
+(defun gnus-agent-save-group-info (method group active)
+  (when (gnus-agent-method-p method)
+    (let* ((gnus-command-method method)
+	   (file (gnus-agent-lib-file "active")))
+      (gnus-make-directory (file-name-directory file))
+      (nnheader-temp-write file
+	(insert-file-contents file)
+	(goto-char (point-min))
+	(when (re-search-forward (concat "^" (regexp-quote group) " ") nil t)
+	  (gnus-delete-line))
+	(insert group " " (number-to-string (cdr active)) " "
+		(number-to-string (car active)) "\n")))))
+
 (defun gnus-agent-group-path (group)
   "Translate GROUP into a path."
   (if nnmail-use-long-file-names
@@ -577,7 +629,7 @@ the actual number of articles toggled is returned."
 ;;;
 
 (defun gnus-agent-fetch-articles (group articles)
-  "Fetch ARTICLES from GROUP and put them into the agent."
+  "Fetch ARTICLES from GROUP and put them into the Agent."
   (when articles
     ;; Prune off articles that we have already fetched.
     (while (and articles
@@ -610,7 +662,7 @@ the actual number of articles toggled is returned."
 		  (insert-buffer-substring nntp-server-buffer)))
 	      (copy-to-buffer nntp-server-buffer (point-min) (point-max))
 	      (setq pos (nreverse pos)))))
-	;; Then save these articles into the agent.
+	;; Then save these articles into the Agent.
 	(save-excursion
 	  (set-buffer nntp-server-buffer)
 	  (while pos
@@ -811,12 +863,15 @@ the actual number of articles toggled is returned."
 	groups group gnus-command-method)
     (save-excursion
       (while methods
-	(setq gnus-command-method (car methods)
-	      groups (gnus-groups-from-server (pop methods)))
-	(gnus-agent-with-fetch
-	  (while (setq group (pop groups))
-	    (when (<= (gnus-group-level group) gnus-agent-handle-level)
-	      (gnus-agent-fetch-group-1 group gnus-command-method)))))
+	(setq gnus-command-method (car methods))
+	(when (or (gnus-server-opened gnus-command-method)
+		  (gnus-open-server gnus-command-method))
+	  (setq groups (gnus-groups-from-server (car methods)))
+	  (gnus-agent-with-fetch
+	    (while (setq group (pop groups))
+	      (when (<= (gnus-group-level group) gnus-agent-handle-level)
+		(gnus-agent-fetch-group-1 group gnus-command-method)))))
+	(pop methods))
       (gnus-message 6 "Finished fetching articles into the Gnus agent"))))
 
 (defun gnus-agent-fetch-group-1 (group method)
@@ -827,7 +882,8 @@ the actual number of articles toggled is returned."
 	gnus-use-cache articles score arts
 	category predicate info marks score-param)
     ;; Fetch headers.
-    (when (and (setq articles (gnus-list-of-unread-articles group))
+    (when (and (or (gnus-active group) (gnus-activate-group group))
+	       (setq articles (gnus-list-of-unread-articles group))
 	       (gnus-agent-fetch-headers group articles))
       ;; Parse them and see which articles we want to fetch.
       (setq gnus-newsgroup-dependencies
@@ -1138,7 +1194,7 @@ The following commands are available:
 
 (defun gnus-agent-high-scored-p ()
   "Say whether an article has a high score or not."
-  (> gnus-score gnus-agent-low-score))
+  (> gnus-score gnus-agent-high-score))
 
 (defun gnus-category-make-function (cat)
   "Make a function from category CAT."
@@ -1210,70 +1266,103 @@ The following commands are available:
 	 (setq gnus-agent-current-history
 	       (setq history (gnus-agent-history-buffer))))
 	(goto-char (point-min))
-	(while (not (eobp))
-	  (skip-chars-forward "^\t")
-	  (if (> (read (current-buffer)) day)
-	      ;; New article; we don't expire it.
-	      (forward-line 1)
-	    ;; Old article.  Schedule it for possible nuking.
-	    (while (not (eolp))
-	      (setq sym (let ((obarray expiry-hashtb))
-			  (read (current-buffer))))
-	      (if (boundp sym)
-		  (set sym (cons (cons (read (current-buffer)) (point))
-				 (symbol-value sym)))
-		(set sym (list (cons (read (current-buffer)) (point)))))
-	      (skip-chars-forward " "))
-	    (forward-line 1)))
-	;; We now have all articles that can possibly be expired.
-	(mapatoms
-	 (lambda (sym)
-	   (setq group (symbol-name sym)
-		 articles (sort (symbol-value sym) 'car-less-than-car)
-		 low (car (gnus-active group))
-		 info (gnus-get-info group)
-		 unreads (ignore-errors (gnus-list-of-unread-articles group))
-		 marked (nconc (gnus-uncompress-range
-				(cdr (assq 'ticked (gnus-info-marks info))))
-			       (gnus-uncompress-range
-				(cdr (assq 'dormant (gnus-info-marks info)))))
-		 nov-file (gnus-agent-article-name ".overview" group))
-	   (gnus-message 5 "Expiring articles in %s" group)
-	   (set-buffer overview)
-	   (erase-buffer)
-	   (when (file-exists-p nov-file)
-	     (insert-file-contents nov-file))
-	   (goto-char (point-min))
-	   (while (setq elem (pop articles))
-	     (setq article (car elem))
-	     (when (or (null low)
-		       (< article low)
-		       (and (not (memq article unreads))
-			    (not (memq article marked))))
-	       ;; Find and nuke the NOV line.
-	       (while (and (not (eobp))
-			   (< (setq art (read (current-buffer))) article))
-		 (forward-line 1))
-	       (if (or (eobp)
-		       (/= art article))
-		   (beginning-of-line)
-		 (gnus-delete-line))
-	       ;; Nuke the article.
-	       (when (file-exists-p (setq file (gnus-agent-article-name
-						(number-to-string article)
-						group)))
-		 (delete-file file))
-	       ;; Schedule the history line for nuking.
-	       (push (cdr elem) histories)))
-	   (write-region (point-min) (point-max) nov-file nil 'silent))
-	 expiry-hashtb)
-	(set-buffer history)
-	(setq histories (nreverse (sort histories '<)))
-	(while histories
-	  (goto-char (pop histories))
-	  (gnus-delete-line))
-	(gnus-agent-save-history)
-	(gnus-agent-close-history)))))
+	(when (> (buffer-size) 1)
+	  (goto-char (point-min))
+	  (while (not (eobp))
+	    (skip-chars-forward "^\t")
+	    (if (> (read (current-buffer)) day)
+		;; New article; we don't expire it.
+		(forward-line 1)
+	      ;; Old article.  Schedule it for possible nuking.
+	      (while (not (eolp))
+		(setq sym (let ((obarray expiry-hashtb))
+			    (read (current-buffer))))
+		(if (boundp sym)
+		    (set sym (cons (cons (read (current-buffer)) (point))
+				   (symbol-value sym)))
+		  (set sym (list (cons (read (current-buffer)) (point)))))
+		(skip-chars-forward " "))
+	      (forward-line 1)))
+	  ;; We now have all articles that can possibly be expired.
+	  (mapatoms
+	   (lambda (sym)
+	     (setq group (symbol-name sym)
+		   articles (sort (symbol-value sym) 'car-less-than-car)
+		   low (car (gnus-active group))
+		   info (gnus-get-info group)
+		   unreads (ignore-errors (gnus-list-of-unread-articles group))
+		   marked (nconc (gnus-uncompress-range
+				  (cdr (assq 'ticked (gnus-info-marks info))))
+				 (gnus-uncompress-range
+				  (cdr (assq 'dormant
+					     (gnus-info-marks info)))))
+		   nov-file (gnus-agent-article-name ".overview" group))
+	     (gnus-agent-load-alist group)
+	     (gnus-message 5 "Expiring articles in %s" group)
+	     (set-buffer overview)
+	     (erase-buffer)
+	     (when (file-exists-p nov-file)
+	       (insert-file-contents nov-file))
+	     (goto-char (point-min))
+	     (setq article 0)
+	     (while (setq elem (pop articles))
+	       (setq article (car elem))
+	       (when (or (null low)
+			 (< article low)
+			 gnus-agent-expire-all
+			 (and (not (memq article unreads))
+			      (not (memq article marked))))
+		 ;; Find and nuke the NOV line.
+		 (while (and (not (eobp))
+			     (or (not (numberp
+				       (setq art (read (current-buffer)))))
+				 (< art article)))
+		   (if (file-exists-p
+			(gnus-agent-article-name
+			 (number-to-string article) group))
+		       (forward-line 1)
+		     ;; Remove old NOV lines that have no articles.
+		     (gnus-delete-line)))
+		 (if (or (eobp)
+			 (/= art article))
+		     (beginning-of-line)
+		   (gnus-delete-line))
+		 ;; Nuke the article.
+		 (when (file-exists-p (setq file (gnus-agent-article-name
+						  (number-to-string article)
+						  group)))
+		   (delete-file file))
+		 ;; Schedule the history line for nuking.
+		 (push (cdr elem) histories)))
+	     (write-region (point-min) (point-max) nov-file nil 'silent)
+	     ;; Delete the unwanted entries in the alist.
+	     (setq gnus-agent-article-alist
+		   (sort gnus-agent-article-alist 'car-less-than-car))
+	     (let* ((alist gnus-agent-article-alist)
+		    (prev (cons nil alist))
+		    (first prev))
+	       (while (and alist
+			   (<= (caar alist) article))
+		 (if (or (not (cdar alist))
+			 (not (file-exists-p
+			       (gnus-agent-article-name
+				(number-to-string
+				 (caar alist))
+				group))))
+		     (setcdr prev (setq alist (cdr alist)))
+		   (setq prev alist
+			 alist (cdr alist))))
+	       (setq gnus-agent-article-alist (cdr first)))
+	     (gnus-agent-save-alist group))
+	   expiry-hashtb)
+	  (set-buffer history)
+	  (setq histories (nreverse (sort histories '<)))
+	  (while histories
+	    (goto-char (pop histories))
+	    (gnus-delete-line))
+	  (gnus-agent-save-history)
+	  (gnus-agent-close-history))
+	(gnus-message 4 "Expiry...done")))))
 
 ;;;###autoload
 (defun gnus-agent-batch ()
