@@ -1,7 +1,7 @@
 ;;; gnus-eform.el --- a mode for editing forms for Gnus
 ;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
 
-;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
+;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
 
 ;; This file is part of GNU Emacs.
@@ -48,6 +48,7 @@
 
 ;;; Internal variables
 
+(defvar gnus-edit-form-buffer "*Gnus edit form*")
 (defvar gnus-edit-form-done-function nil)
 
 (defvar gnus-edit-form-mode-map nil)
